% Author:   Thomas Oliver Jelly & Angela Busse
% Location: University of Glasgow & Melbourne
% Purpose:  Step 5 of the MARS algorithm

function [hmap] = mars_stepfive(hmap,filt,nsurf)

  % Number of modes and elliptic filter
  mmax = filt.cutoff; 
  nmax=(hmap.Ly/hmap.Lx)*filt.cutoff;
  set_ellipse=1.0;

  % Apply circular Fourier filter to heightmap
  fft_height=transpose(fft2(hmap.raw(:,:,nsurf)'));

  %
  M=size(fft_height,1);
  N=size(fft_height,2);
  %
  if(nargout>=2)
      mask=zeros(M,N);
  end
  % first part of mask
  for i=1:1:mmax+1
      for j=1:1:nmax+1
          if i==1 && j==1 
              continue
          end
          if set_ellipse==1
              kel=((i-1)/mmax)^2+((j-1)/nmax)^2;
              if kel > 1
                  continue
              end
          end       
          if(nargin>=2);
              mask(i,j)=1;
              if(i-1>0 && j-1>0)
                  mask(M-i+2,N-j+2)=1;
              elseif(i>1)
                  mask(M-i+2,j)=1;
              elseif(j>1)
                  mask(i,N-j+2)=1;
              end
          end
      end
  end
  % second part of mask
  for i=0:-1:-mmax+1
      for j=2:1:nmax+1
          if set_ellipse==1
              kel=((i-1)/mmax)^2+((j-1)/nmax)^2;
              if kel > 1
                  continue
              end
          end
          if(nargin>=2);
              mask(M+i,j)=1;
              mask(-i+2,N-j+2)=1;
          end
      end
  end
 tmp = transpose(ifft2(mask.*fft_height));

 % Re-scale heightmap to have unit variance
 factor = 1.d0./std(tmp(:));
 hmap.filtered(1:M,1:N,nsurf) = factor*tmp(:,:)';
 clear tmp
