% Author:   Thomas Oliver Jelly & Angela Busse
% Location: University of Glasgow & Melbourne
% Purpose:  Step 4 of the MARS algorithm

function [hmap]= mars_stepfour(nls,hmap,nsurf);

  % Get sizes
  N = hmap.Nx; 
  M = hmap.Ny;
  [n,m] = size(nls.coeff);

  % Generate [N x M] Gaussian random number matrix
  eta = randn(N,M); 

  % Subtract mean
  eta = eta - mean(eta(:));

  % Normalise to unit variance
  fac = 1.d0./std(eta(:));
  eta = fac.*eta;
 
  % Enforce periodic indices using mod function
  ko= mod(0:(N+n)-1,N)+1;
  lo= mod(0:(M+m)-1,M)+1;

  % Evaluate linear transformation to obtain heightmap
  tmp = zeros(size(eta));
  for i=1:N
    for j=1:M
      for k=1:n
        for l=1:m
          tmp(i,j) = tmp(i,j) + nls.coeff(k,l)*eta(ko(i+k),lo(j+l));
        end 
      end 
    end
  end

  % Subtract mean roughness plane
  tmp = tmp - mean(tmp(:));

  % Normalise to unit variance
  fac = 1.d0./std(tmp(:));
  tmp = fac.*tmp;

  % Copy tmp into hmap
  hmap.raw(1:N,1:M,nsurf) = tmp(:,:); 
  clear tmp;
