% Author:   Thomas Oliver Jelly & Angela Busse
% Location: University of Glasgow & Melbourne
% Purpose:  Step 2 of the MARS algorithm

function [corr]= mars_steptwo(corr);

  % n & m based on 0.1 cutoff criterion
  n0p1 = corr.n;
  m0p1 = corr.m;

  % Calculate n and m based on current value of gamma
  n = floor(corr.n*(log(corr.gamma)/-2.3d0));
  m = floor(corr.m*(log(corr.gamma)/-2.3d0));

  % Assemble [n x m] correlation function 
  tmp = zeros(n,m);
  for p=0:n-1
    for q=0:m-1
      arg = -2.3d0*sqrt((p/n0p1)^2.d0+(q/m0p1)^2.d0); 
      tmp(p+1,q+1) = exp(arg);
    end
  end

  % Copy tmp into corr.rhs and clear
  corr.rhs = tmp;
  clear tmp
