  % Author:      Thomas O. Jelly & Angela Busse
  % Location(s): University of Melbourne & University of Glasgow
  % Purpose:     MATLAB implementation of MARS algorithm 

  % Tidy up
  clear all;  
  close all; 
  clc

  % Start: Initialise MARS algorithm
  addpath('lib')
  mars_input;
  fprintf('\nMARS algorithm has been initialised.\n')

  % STEP 1: Specify tile size and heightmap dimensions
  [hmap,nls] = mars_stepone(hmap,nls,corr);

  % Do coefficients exist?
  if isequal(exist(strcat('archive/',nls.fname),'file'),2)

    % YES -> Load the coefficients from the archive
    fprintf(['\n  Coefficients exist for (n,m) = ',... 
            '(',num2str(corr.n),',',num2str(corr.m),')\n'])
    tmp = load(strcat('archive/',nls.fname));
    nls.coeff = tmp.nls.coeff;
    clear tmp
  else

    % NO -> Solve non-linear system until convergence
    fprintf(['\n  Coefficients do not exist for (n,m) = ',... 
            '(',num2str(corr.n),',',num2str(corr.m),')'...,
             '\n  Solving non-linear system...\n'])

    % Conditional loop for STEP 2, STEP 3
    nls.success = 0;
    corr.gamma = 0.1d0;
    while (nls.success ~= 1)

      % STEP 2: Assemble the correlation function
      [corr] = mars_steptwo(corr);

      % STEP 3: Solve non-linear system of equations
      [nls] = mars_stepthree(corr,nls);

      % Converged?
      if nls.success ~= 1
        % NO -> Reduce cutoff parameters
        corr.gamma = corr.gamma*0.5d0;
      else
        % YES -> Save coefficients in archive folder
        save(strcat('archive/',nls.fname),'nls');
      end 
    end
  end 

  % Main loop to generate heightmaps
  for i = 1:hmap.N; 
    % STEP 4: Evaluate linear transformation
    [hmap] = mars_stepfour(nls,hmap,i);

    % STEP 5: Apply (optional) two-dimensional Fourier filter
    if filt.option == 1
      % YES -> Apply Fourier filter
      [hmap] = mars_stepfive(hmap,filt,i);
      % NO -> Do not apply Fourier filter
      else
      hmap.filtered(:,:,i) = hmap.raw(:,:,i);
    end

    % Print message to screen
    fprintf(['\n  - Generated ',num2str(i),'/',num2str(hmap.N),...
             ' heightmaps\n'])
  end

  % End
  fprintf('\nMARS algorithm has exited sucessfully.\n\n')
